/* HyperCard XFCN to produce the Context display for browsing in
 * free-text databases ...
 *
 * called as     getContexts ( N, M, W, O, P, T)
 *
 * N = ptr record number of first line to display
 * M = how many lines to get
 * W = width in characters of each line
 * O = offset of key word from start of each line
 * P = ptr file refNum
 * T = text file refNum
 *
 * The results are returned as a nice printable zero-terminated string.
 * Each line has control characters (tabs, returns, line feeds, etc.)
 * turned into spaces, so that they will print out nicely on the screen.
 *
 * If a mistake is detected, a null string is returned and the function
 * also emits a beep....
 *
 * Function is stored as XFCN number 2401, named "getContexts"....
 *
 * 871208 ^z
 */

#include <MacTypes.h>
#include <FileMgr.h>
#include <OSUtil.h>
#include <HyperXCmd.h>
#include <proto.h>


pascal void main (XCmdBlockPtr paramPtr);
void getFilteredText (long bytes, long start, int refNum,
	unsigned char *ans);
long getThePtr (long ptrNum, int refNum);
void complain (XCmdBlockPtr paramPtr);


pascal void main (paramPtr)
  XCmdBlockPtr paramPtr;
  {
	int ptrFileNum, textFileNum, lines, width, offset, i;
	long ptrNum, ptr;
	Handle answer;
	
	if (paramPtr->paramCount != 6)
	  {
	  	complain (paramPtr);
	  	return;
	  }
	 
	ptrNum = atol (*(paramPtr->params[0]));
	lines = atol (*(paramPtr->params[1]));
	width = atol (*(paramPtr->params[2]));
	offset = atol (*(paramPtr->params[3]));
	ptrFileNum = atol (*(paramPtr->params[4]));
	textFileNum = atol (*(paramPtr->params[5]));
	
	if (ptrNum < 0 || lines <= 0 || width <= 0 || offset < -100
		|| offset > width + 100 || ptrFileNum == 0 || textFileNum == 0)
	  {
	  	complain (paramPtr);
	  	return;
	  }
	
	answer = NewHandle (lines * (width + 1) + 1);
	
	for (i = 0; i < lines; ++i)
	  {
		ptr = getThePtr (ptrNum + i, ptrFileNum);
		if (ptr >= 0)
			getFilteredText (width, ptr - offset, textFileNum,
						(unsigned char *)*answer + i * (width + 1));
		*(*answer + (i + 1) * (width + 1) - 1) = '\r';
	  }
		
	*(*answer + lines * (width + 1)) = '\0';
	paramPtr->returnValue = answer;
	return;
  }


/* function to fetch the requested chunk of text from the file ... fill
 * in with blanks if try to fetch from before the beginning of the
 * file or after the end of the file ... filter all control characters
 * by turning them into spaces ....
 */

void getFilteredText (bytes, start, refNum, ans)
  long bytes, start;
  int refNum;
  unsigned char *ans;
  {
	int i = 0;
	long origbytes;
	
	origbytes = bytes;
	if (start < 0)
		for (i = 0; i < -start; ++i)
			ans[i] = ' ';
	bytes -= i;
	SetFPos (refNum, fsFromStart, start + i);
	FSRead (refNum, &bytes, ans + i);
	if (bytes + i < origbytes)
		for (i += bytes; i < origbytes; ++i)
			ans[i] = ' ';
	for (i = 0; i < origbytes; ++i)
	  	if (ans[i] < 32 || ans[i] == 127)
	  		ans[i] = ' ';
	
	return;
  }


/* function to fetch the value of the nth ptr from file ptrFileNum ...
 * return illegal value (-1) for result if something goes wrong....
 */

long getThePtr (n, ptrFileNum)
  long n;
  int ptrFileNum;
  {
	long bytes = sizeof (long), result;
	
	if (SetFPos (ptrFileNum, fsFromStart, n * sizeof (long)) != noErr ||
			FSRead (ptrFileNum, &bytes, &result) != noErr)
		return (-1);

	return (result);
  }



/* function to beep and set the return string to null (= "")
 */

void complain (paramPtr)
  XCmdBlockPtr paramPtr;
  {
	Handle answer;
	
  	SysBeep (10);
	answer = NewHandle (1);
	**answer = '\0';
	paramPtr->returnValue = answer;
	return;
  }



/* function to convert alphabetic string to a long integer ... from LSC
 * library.... simplified to avoid using isspace() & isdigit() .... */

long atol (s)
  register char *s;
  {
	register char signflag = 0;
	register long r = 0;

	while ((*s == ' '))
		s++;
		
	if (*s == '-')
	  {
		signflag = 1;
		s++;
	  }
	else if (*s == '+')
	 	s++;

	while (*s >= '0' && *s <= '9') 
		r = r * 10 + (*s++ - '0');
	
	return (signflag ? -r : r);
}

